-- $Id: invasion-liron.lua 3313 2018-03-27 03:31:39Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

newChat {
  id = 'welcome',
  text = [[#LIGHT_GREEN#*Out of nowhere, a Thunderlord appears in front of you.*#LAST#
@playername@!  Thank Manwë, I have finally found you.  I am Liron, rider of Tolan.  I am sent by Turgon, King of Gondolin to request your urgent aid.  Gondolin is being invaded as we speak; we need your help now or all will be lost.  I can carry you back to Gondolin, but we must go now.  Will you come?]],
  answers = {
    { [[That sounds bad.  Certainly I'll help.]], jump='go' },
    { [[Yyyeah, I'm kinda busy at the moment, sorry.]], jump='second-chance' },
  },
}

newChat {
  id = 'second-chance',
  text = [[Are you sure?  If we do not go right now, Gondolin will be destroyed.]],
  answers = {
    { [[Okay, you've convinced me.]], jump='go' },
    { [[Meh, never liked Gondolin anyway.]], jump='fail' },
  },
}

newChat {
  id = 'fail',
  text = [[Then I have come in vain.  Turgon overestimated you.  I will return to Gondolin alone and will no doubt die there.  May you be doomed for your cowardice! #LIGHT_GREEN#*Liron and his eagle vanish.*#LAST#]],
  answers = {
    {
      [[...In restrospect, that may have been a bad idea.]],
      action = function(npc, player)
	-- Do this manually, so as not to frob the quest dialogs.
	local q = player:hasQuest('invasion-of-gondolin')
	q.objectives['snubbed-liron'] = Quest.FAILED
	-- This will destroy Gondolin for us.
	player:setQuestStatus('invasion-of-gondolin', Quest.FAILED)
      end,
    },
  },
}

local function go_action(npc, player)
  player:setQuestStatus('invasion-of-gondolin', Quest.COMPLETED, 'followed-liron')
  if game.zone and game.zone.short_name == 'town-gondolin' then
    local spot = game.level:pickSpot{type='quest', subtype='invasion-liron-entry'}
    player:move(spot.x, spot.y, true)
  else
    game:changeLevel(1, 'town-gondolin', { entry_point={type='quest', subtype='invasion-liron-entry'} })
  end
  -- Manually invoke the Turgon chat.
  local Chat = require 'engine.Chat'
  local Map = require 'mod.class.Map'
  local npc = game.level.map(player.x, player.y-1, Map.TRAP)
  local chat = Chat.new('turgon', npc, player, {npc=npc, player=player})
  chat:invoke('give-invasion')
end

newChat {
  id = 'go',
  text = [[You have made the right decision, @playername@.  Quickly, jump on Tolan!]],
  answers = {
    {
      [[Let's go.]],
      action = go_action,
      cond = function(npc, player)
	return player.descriptor.race ~= 'Thunderlord'
      end
    },
    {
      [[...Y'know, I #{italic}#do#{normal}# have my own eagle.  Why don't I just follow you?]],
      action = go_action,
      cond = function(npc, player)
	return player.descriptor.race == 'Thunderlord'
      end
    },
    {
      [[Wait, we're #{italic}#in#{normal}# Gondolin.  Couldn't we just walk-]],
      action = go_action,
      cond = function(npc, player)
	return game.zone and game.zone.short_name == 'town-gondolin'
      end,
    },
  },
}

return 'welcome'
