-- $Id: eol-the-dark-elf.lua 3285 2018-03-11 22:48:30Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

id = 'eol-the-dark-elf'
name = 'Eöl the Dark Elf'
danger_level = 30	-- cf. src/tables.c line 3946
reward = 'something'	-- populated by on_status_change() below

desc = function(self, who)
  if self:isFailed() then
    return [[You were unable to defeat Eöl the Dark Elf; you fled, allowing him to escape.]]
  elseif self:isEnded() then
    return ([[You slew Eöl the Dark Elf to prevent him from entering Gondolin.  As a reward, Turgon gave you %s.]]):format(self.reward)
  elseif self:isCompleted() then
    return [[You have slain Eöl the Dark Elf to prevent him from entering Gondolin.  You should return to the Tower of the King and tell Turgon.]]
  else
    return [[Eöl the Dark Elf has been spotted trying to gain entrance to Gondolin by way of the eastern mountains.  You have been sent by Turgon to stop him, by force if necessary.]]
  end
end

on_status_change = function(self, who, status, sub)
  if status == self.DONE then
    local Object = require 'mod.class.Object'
    Object.resolve_flags.force_ego = 'MAGI_LITE'
    local o = game.zone:makeEntityByName(game.level, 'object', 'DWARVEN_LANTERN')
    if o then
      o:identify(true, who)
      o:setFoundHere {
	type = 'custom',
	custom = 'You received %s as a reward for slaying Eöl the Dark Elf.',
      }
      self.reward = o:getName()
      game.logPlayer(who, 'Turgon gives you:  %s', o:getName())
      who:addOrDropObject(who.INVEN_INVEN, o, nil, nil, {vocal_drop=true})
      who:sortInven()
    end
  end
end

on_quest_mon_die = function(self, who, src)
  -- Only interested in kills in the quest zone.
  if game.zone.short_name ~= 'quest-eol-the-dark-elf' then return end
  -- Only interested in kill of Eol.
  if who.define_as == 'EOL_THE_DARK_ELF' then
    game.log('#YELLOW#Such a sad end...#LAST#')
    game.player:setQuestStatus('eol-the-dark-elf', self.COMPLETED)
  end
end
