-- $Id: music.lua 2860 2017-05-07 23:06:25Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- -----------------------------------------------------------------------
-- WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
-- -----------------------------------------------------------------------
-- This file was generated automatically by the 'gen_spells' Perl script.
-- DO NOT MODIFY THIS FILE DIRECTLY, as any changes will be lost the next
-- time the file is generated (with the possible exception of lines between
-- 'BEGIN VERBATIM' and 'END VERBATIM' markers, if supported by the script).
-- -----------------------------------------------------------------------
-- WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
-- -----------------------------------------------------------------------

-- Spell talents generated from T2's s_music.lua

-- BEGIN VERBATIM
-- Spend mana for a sustained Music spell, and cancel the spell when we run
-- out.
local function mana_sustain(who, t)
  who:incMana(-spell.mana_cost(who, t))
  if who:getMana() <= 0 then
    game.logPlayer(who, 'You stop your spell.')
    who:forceUseTalent(t.id, { ignore_energy=true })
  end
end

-- Cancel all currently sustained Music spells.  This will be called in the
-- activate() method of sustained Music spells, which is called before the
-- spell is added to sustain_talents[], so we don't have to worry about
-- dodging the newly added spell.
local function cancel_other_music(who)
  for t_id, _ in pairs(who.sustain_talents) do
    local t = who:getTalentFromId(t_id)
    if t.type == 'spell/music' then
      who:forceUseTalent(t.id, { ignore_energy=true })
    end
  end
end

local sustain_sfx = [[

You can only have one sustained song active at a time.  Starting a sustained song will stop all other currently active sustained songs.]]
-- END VERBATIM

local Entity = require 'engine.Entity'

newTalentType {
  name = 'Music spells',
  type = 'spell/music',
  description = 'Spells from the Music school',
}

newTalent {
  name = [[Holding Pattern(I)]],
  short_name = 'MUSIC_HOLD',
  display_entity = Entity.new { image='talents/holding_pattern.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 1,
  mana = spell.mana_cost,
  min_mana = 1,
  max_mana = 10,
  base_fail = 20,
  blind_cast = true,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 1,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      local pow = 10 + spell.get_spell_level(self, t, 100)
      spell.los_spell(self, t, DamageType.OLD_SLOW, pow)
      mana_sustain(self, t)
    end,
  },
  info = function(self, t)
    local pow = 10 + spell.get_spell_level(self, t, 100)
    local mana = spell.mana_cost(self, t)
    local sfx = spell.spell_skill_desc(self, t)
    local ret = [[Attempts to slow down all monsters listening to the song (with power %d) for as long as you sing, at a cost of %d mana per turn.  The spell's power increases%s]]
    return ret:format(pow, mana, sfx) .. sustain_sfx
  end,
}

-- Should be unnecessary if we make all the ongoing songs sustained talents.
--[=[
newTalent {
  name = [[Stop singing(I)]],
  short_name = 'MUSIC_STOP',
  display_entity = Entity.new { image='talents/stop_singing.png' },
  type = 'spell/music',
  mode = 'activated',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 1,
  mana = spell.mana_cost,
  min_mana = 0,
  max_mana = 0,
  base_fail = -400,
  blind_cast = true,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 1,
  points = 0,
  range = 18,
  direct_hit = true,
  action = function(self, t)
			player.start_lasting_spell(0)
			return TRUE
	end,
  info = [[Stops the current song, if any.]],
  info_aux = function(self, t)
			return ""
	end,
}
]=]

newTalent {
  name = [[Song of the Sun(I)]],
  short_name = 'MUSIC_SONG_SUN',
  display_entity = Entity.new { image='talents/song_of_the_sun.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 1,
  mana = spell.mana_cost,
  min_mana = 1,
  max_mana = 1,
  base_fail = 20,
  blind_cast = true,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 1,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      self:setEffect(self.EFF_LIGHT, 5, {})
      mana_sustain(self, t)
    end,
  },
  info = function(who, t)
    local mana = spell.mana_cost(self, t)
    local ret = [[Increases your light radius by 2 while you sing, at a cost of %d mana per turn.]]
    return ret:format(mana) .. sustain_sfx
  end,
}

newTalent {
  name = [[Blow(I)]],
  short_name = 'MUSIC_BLOW',
  display_entity = Entity.new { image='talents/blow.png' },
  type = 'spell/music',
  mode = 'activated',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 4,
  mana = spell.mana_cost,
  min_mana = 3,
  max_mana = 30,
  base_fail = 20,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 1,
  points = 0,
  range = 18,
  direct_hit = true,
  action = function(self, t)
    local r = 1 + spell.get_spell_level(self, t, 12, 0)
    local dd, ds = t.dam_dice(self, t)
    local dam = rng.dice(dd, ds)
    spell.ball_spell(self, t, DamageType.SOUND, dam, r, self.x, self.y)
    return true
  end,
  info = function(self, t)
    local dd, ds = t.dam_dice(self, t)
    local r = 1 + spell.get_spell_level(self, t, 12, 0)
    local dtf = util.colorize_damtype(DamageType.SOUND)
    local sfx = spell.spell_skill_desc(self, t)
    local ret = [[Produces a powerful, blowing, sound all around you, doing %dd%d %s damage in a radius of %d.  Damage and radius will increase%s]]
    return ret:format(dd, ds, dtf, r, sfx)
  end,
  -- BEGIN VERBATIM
  dam_dice = function(self, t)
    local dd = 2 + spell.get_spell_level(self, t, 10, 0)
    local ds = 4 + spell.get_spell_level(self, t, 40, 0)
    return dd, ds
  end
  -- END VERBATIM
}

newTalent {
  name = [[Illusion Pattern(II)]],
  short_name = 'MUSIC_ILLUSION',
  display_entity = Entity.new { image='talents/illusion_pattern.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 5,
  mana = spell.mana_cost,
  min_mana = 2,
  max_mana = 15,
  base_fail = 30,
  blind_cast = true,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 2,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      local pow = 10 + spell.get_spell_level(self, t, 100)
      spell.los_spell(self, t, DamageType.OLD_CONF, pow)
      mana_sustain(self, t)
    end,
  },
  info = function(self, t)
    local pow = 10 + spell.get_spell_level(self, t, 100)
    local mana = spell.mana_cost(self, t)
    local sfx = spell.spell_skill_desc(self, t)
    local ret = [[Attempts to confuse all monsters listening to the song (with power %d) for as long as you sing, at a cost of %d mana per turn.  The spell's power increases%s]]
    return ret:format(pow, mana, sfx) .. sustain_sfx
  end,
}

newTalent {
  name = [[Flow of Life(II)]],
  short_name = 'MUSIC_FLOW_LIFE',
  display_entity = Entity.new { image='talents/flow_of_life.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 7,
  mana = spell.mana_cost,
  min_mana = 5,
  max_mana = 30,
  base_fail = 35,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 2,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      local amt = 7 + spell.get_spell_level(self, t, 100)
      self:heal(amt)
      mana_sustain(self, t)
    end,
  },
  info = function(self, t)
    local amt = 7 + spell.get_spell_level(self, t, 100)
    local mana = spell.mana_cost(self, t)
    local sfx = spell.spell_skill_desc(self, t)
    local ret = [[Heals %d hit points per turn for as long as you sing, at a cost of %d mana per turn.  The amount healed increases%s]]
    return ret:format(amt, mana, sfx) .. sustain_sfx
  end,
}

newTalent {
  name = [[Stun Pattern(IV)]],
  short_name = 'MUSIC_STUN',
  display_entity = Entity.new { image='talents/stun_pattern.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 10,
  mana = spell.mana_cost,
  min_mana = 3,
  max_mana = 25,
  base_fail = 45,
  blind_cast = true,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 4,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      local pow = 10 + spell.get_spell_level(self, t, 90)
      spell.los_spell(self, t, DamageType.STUN, pow)
      mana_sustain(self, t)
    end,
  },
  info = function(self, t)
    local pow = 10 + spell.get_spell_level(self, t, 90)
    local mana = spell.mana_cost(self, t)
    local sfx = spell.spell_skill_desc(self, t)
    local ret = [[Attempts to stun all monsters listening to the song (with power %d) for as long as you sing, at a cost of %d mana per turn.  The spell's power increases%s]]
    return ret:format(pow, mana, sfx) .. sustain_sfx
  end,
}

newTalent {
  name = [[Heroic Ballad(II)]],
  short_name = 'MUSIC_HEROIC',
  display_entity = Entity.new { image='talents/heroic_ballad.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 10,
  mana = spell.mana_cost,
  min_mana = 4,
  max_mana = 14,
  base_fail = 45,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 2,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      local lev = spell.get_spell_level(self, t)
      self:setEffect(self.EFF_HEROISM, 5, {})
      if lev >= 10 then self:setEffect(self.EFF_SUPERHEROISM, 5, {}) end
      if lev >= 20 then self:setEffect(self.EFF_ACCURATE_STRIKES, 5, {}) end
      if lev >= 25 then self:setEffect(self.EFF_RES_CHAOS_CONF, 5, {}) end
    end,
  },
  info = function(self, t)
    local ff = util.colorize_damtype(DamageType.FEAR)
    local chf = util.colorize_damtype(DamageType.CHAOS)
    local cnf = util.colorize_damtype(DamageType.CONFUSION)
    local mana = spell.mana_cost(self, t)
    local ret = [[Confers heroism (+12 to-hit and resist %s) while you sing, at a cost of %d mana per turn.  At level 10 it also confers super-heroism (+24 to-hit and -10 AC).  At level 20 it makes you even more deadly, conferring an additional +15 to-hit and to-damage.  At level 25 it also confers resistance to %s and %s.]]
    return ret:format(ff, mana, chf, cnf) .. sustain_sfx
  end,
}

newTalent {
  name = [[Gush of Wind(II)]],
  short_name = 'MUSIC_WIND',
  display_entity = Entity.new { image='talents/gush_of_wind.png' },
  type = 'spell/music',
  mode = 'activated',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 14,
  mana = spell.mana_cost,
  min_mana = 15,
  max_mana = 45,
  base_fail = 30,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 2,
  points = 0,
  range = 18,
  direct_hit = true,
  action = function(self, t)
    local pow = 10 + spell.get_spell_level(self, t, 40, 0)
    local r = 1 + spell.get_spell_level(self, t, 12, 0)
    spell.ball_spell(self, t, DamageType.TELEPORT, pow, r, self.x, self.y)
    return true
  end,
  info = function(self, t)
    local pow = 10 + spell.get_spell_level(self, t, 40, 0)
    local r = 1 + spell.get_spell_level(self, t, 12, 0)
    local sfx = spell.spell_skill_desc(self, t)
    local ret = [[Produces a outgoing gush of wind in a radius of %d around you, teleporting monsters up to %d away.  Area of effect and distance teleported will increase%s]]
    return ret:format(r, pow, sfx)
  end,
}

newTalent {
  name = [[Horns of Ylmir(III)]],
  short_name = 'MUSIC_YLMIR',
  display_entity = Entity.new { image='talents/horns_of_ylmir.png' },
  type = 'spell/music',
  mode = 'activated',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 20,
  mana = spell.mana_cost,
  min_mana = 25,
  max_mana = 30,
  base_fail = 20,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 3,
  points = 0,
  range = 18,
  direct_hit = true,
  action = function(self, t)
    local r = 2 + spell.get_spell_level(self, t, 10)
    spell.earthquake(self.x, self.y, r)
    return true
  end,
  info = function(self, t)
    local r = 2 + spell.get_spell_level(self, t, 10)
    local ret = [[You produce an earth-shaking sound, triggering a localized earthquake of radius %d around you.]]
    return ret:format(r)
  end,
}

newTalent {
  name = [[Hobbit Melodies(III)]],
  short_name = 'MUSIC_HOBBIT',
  display_entity = Entity.new { image='talents/hobbit_melodies.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 20,
  mana = spell.mana_cost,
  min_mana = 10,
  max_mana = 30,
  base_fail = 70,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 3,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      local ac = 10 + spell.get_spell_level(self, t, 50)
      local spd = 7 + spell.get_spell_level(self, t, 10)
      self:setEffect(self.EFF_EVASION, 5, { power=ac })
      if spell.get_spell_level(self, t) >= 15 then
	self:setEffect(self.EFF_EVASION, 5, { merge='replace', speed=spd })
      end
      mana_sustain(self, t)
    end,
  },
  info = function(self, t)
    local ac = 10 + spell.get_spell_level(self, t, 50)
    local spd = 7 + spell.get_spell_level(self, t, 10)
    local mana = spell.mana_cost(self, t)
    local ret = [[Greatly increases your reflexes while you sing, allowing you to block more melee blows (+%d AC), at a cost of %d mana per turn. At level 15 it also increases your speed by %d while you sing.  AC and speed boosts will increase%s]]
    return ret:format(ac, mana, spd, sfx) .. sustain_sfx
  end,
}

newTalent {
  name = [[Clairaudience(IV)]],
  short_name = 'MUSIC_CLAIRAUDIENCE',
  display_entity = Entity.new { image='talents/clairaudience.png' },
  type = 'spell/music',
  mode = 'sustained',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 25,
  mana = spell.mana_cost,
  min_mana = 15,
  max_mana = 30,
  base_fail = 75,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 4,
  points = 0,
  range = 18,
  direct_hit = true,
  activate = function(who, t)
    cancel_other_music(who)
    return {}
  end,
  deactivate = function(who, t) return true end,
  active_callbacks = {
    on_global_turn = function(self, t)
      who:setEffect(who.EFF_SENSE_MONSTER, 5, {})
      if spell.get_spell_level(self, t) >= 10 then
	local r = 1 + spell.get_spell_level(self, t, 3, 0)
	spell.ball_spell(self, t, DamageType.IDENTIFY, 0, r, self.x, self,y, 1)
      end
      mana_sustain(self, t)
    end,
  },
  info = function(self, t)
    local mana = spell.mana_cost(self, t)
    local r = 1 + spell.get_spell_level(self, t, 3, 0)
    local sfx = spell.spell_skill_desc(self, t)
    local ret = [[Allows you to sense monster minds while you sing, at a cost of %d mana per turn.  At level 10, it will also identify objects on the floor in radius %d around you, and probe monsters in the same radius.  ID/probe radius will increase%s]]
    return ret:format(mana, r, sfx) .. sustain_sfx
  end,
}

newTalent {
  name = [[Ambarkanta(IV)]],
  short_name = 'MUSIC_AMBARKANTA',
  display_entity = Entity.new { image='talents/ambarkanta.png' },
  type = 'spell/music',
  mode = 'activated',
  is_spell = true,
  skill = 'MUSIC',
  skill_level = 25,
  mana = spell.mana_cost,
  min_mana = 70,
  max_mana = 70,
  base_fail = 60,
  stat = 'CHA',
  random_book = 'MUSIC',
  instrument_pval = 4,
  points = 0,
  range = 18,
  direct_hit = true,
  action = function(self, t)
    spell.alter_reality()
    return true
  end,
  info = function(self, t)
    return [[Produces a reality shaking sound that transports you to a nearly identical reality.]]
  end,
}
