-- $Id: zone.lua 2934 2017-06-15 00:29:05Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Dialog = require 'engine.ui.Dialog'

return {
  name = "Thieves' Lair",
  level_scheme = 'fixed',
  max_level = 1,
  level_range = { 5, 5 },
  width = 33,
  height = 23,
  no_level_connectivity = true,
  no_recall_depth = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.Static',
      map = 'quests/bree-thieves',
    }
  },

  on_enter = function(self, _, _, _)
    local spot = game.level:pickSpot{ type='quest', subtype='player-inven' }
    for id, inven in pairs(game.player.inven) do
      repeat
	local try_again = false
	for item, o in ipairs(inven) do
	  local can_remove =
	    not o:check('on_preremoveobject', game.player, inven, true) and
	    not o:on_drop(game.player, true) and
	    not (inven.worn and o:check('on_cantakeoff', game.player, inven, true))
	  if can_remove then
	    local oo = game.player:removeObject(inven, item, true)
	    -- T2 scatters these on the floor near the drop point; we're
	    -- just going to punt and put them all in one pile.
	    game.level.map:addObject(spot.x, spot.y, oo)
	    -- We just modified the inventory table, which will break
	    -- iteration; punt and start again from the top.
	    try_again = true
	    break
	  end
	end
      until #inven == 0 or not try_again
    end
    Dialog:simplePopup('Quest: Thieves', "As you enter the thieves' lair, you are struck on the head from behind and knocked out.  You awake in a prison cell with all of your possessions missing.")
  end
}
