-- $Id: zone.lua 3140 2017-10-16 00:05:49Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

return {
  name = 'Lothlórien',
  level_scheme = 'fixed',
  max_level = 1,
  width = 198,
  height = 66,
  no_level_connectivity = true,
  all_remembered = true,
  all_lited = true,
  day_night = true,
  town = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.PlasmaWilderness',
      map = game.state:varTownDestroyed('lothlorien', 'map'),
    },
    actor = game.state:varTownDestroyed('lothlorien', 'actor_gen', {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 10,
      filters = {
	{ prob=100, mode='any' },
      },
      zone_resolve_flags = { use_ego='ELVEN' },
    }),
  },

  entry_points_from_zone = {
    ['quest-lothlorien-wolves'] = { type='quest', subtype='house' },
    ['quest-lothlorien-spiders'] = { type='quest', subtype='spiders' },
    ['quest-lothlorien-fireproof'] = { type='quest', subtype='mage' },
  },
  entry_point = util.town_wild_entry_point,

  quest_entrances = {
    ['lothlorien-house'] = {
      type='quest', subtype='house', replace={
	[Quest.PENDING] = 'ENTER_LOTHLORIEN_HOUSE_QUEST',
	[Quest.COMPLETED] = 'ENTER_LOTHLORIEN_HOUSE_QUEST',
	-- Default grid is PERMA_WALL, which is what we wanted anyway.
	[Quest.DONE] = { trap='LOTHLORIEN_HOME' },
      },
    },
    ['lothlorien-spiders'] = {
      type='quest', subtype='spiders', replace={
	[Quest.PENDING] = 'ENTER_LOTHLORIEN_SPIDERS_QUEST',
	-- Let the entrance go away after we complete (or fail) the quest
      },
    },
    ['lothlorien-fireproof'] = {
      type='quest', subtype='mage', replace={
	[Quest.PENDING] = 'ENTER_LOTHLORIEN_FIREPROOF_QUEST',
	-- Let the entrance go away after we complete (or fail) the quest
      },
    },
  },
  on_enter = util.juggle_quest_entrances,

  on_leave = util.on_leave_town_wild,
}
