-- $Id: LogDisplay.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local LogDisplay = require 'engine.LogDisplay'

module(..., package.seeall, class.inherit(LogDisplay))

function _M:init(x, y, w, h, max, fontname, fontsize, color, bgcolor)
  LogDisplay.init(self, x, y, w, h, max, fontname, fontsize, color, bgcolor)
  getmetatable(self).__call = _M.call
end

-- Copied from parent.
local urlfind = (lpeg.P"http://" + lpeg.P"https://") * (1-lpeg.P" ")^0
local urlmatch = lpeg.anywhere(lpeg.C(urlfind))

-- Overloaded to handle repeated log messages.
function _M:call(str, ...)
  str = str or ""
  str = str:format(...)
  print("[LOG]", str)
  local tstr = str:toString()
  if self.out_f then
    self.out_f:write(tstr:removeColorCodes())
    self.out_f:write("\n")
  end

  local url = urlmatch:match(tstr)
  if url then
    tstr = tstr:lpegSub(urlfind, "#LIGHT_BLUE##{italic}#"..url.."#{normal}##LAST#")
  end

  if self.log[1] and self.log[1].orig_str == tstr then
    -- Repeated log message; bump the repeat count.
    self.log[1].repeats = self.log[1].repeats + 1
    local sfx = (' (×%d)'):format(self.log[1].repeats)
    -- Clear the cached version of the old log message.  [Can't tell from
    -- the parent code whether this cache is supposed to be keyed by the
    -- entire log entry table or just its 'str' field, so clear both just
    -- to be safe.  The cache itself will be populated in the parent's
    -- display() method.]
    self.cache[self.log[1].str] = nil
    self.cache[self.log[1]] = nil
    self.log[1].str = self.log[1].orig_str .. sfx
  else
    table.insert(self.log, 1, {
      str = tstr,
      orig_str = tstr,
      repeats = 1,
      timestamp = core.game.getTime(),
      url = url,
    })
  end
  while #self.log > self.max_log do
    local old = table.remove(self.log)
    -- As above.
    self.cache[old.str] = nil
    self.cache[old] = nil
  end
  self.max = #self.log
  self.changed = true
end
