-- $Id: EolQuestLevel.lua 3285 2018-03-11 22:48:30Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Map = require 'mod.class.Map'
local T2LevelGenerator = require 'mod.class.generator.map.T2LevelGenerator'

module(..., package.seeall, class.inherit(T2LevelGenerator))

-- We replace the parent :generate() wholesale, because we don't need or
-- want most of its stuff.
-- [cf. quest_eol_gen_hook() in src/q_eol.c]
function _M:generate(lev, old_lev)
  -- Dunno if we need this, but just in case.
  self.level.rating = 0

  -- Fill with perma-wall
  for x = 0, self.map.w - 1 do
    for y = 0, self.map.h - 1 do
      self.map(x, y, Map.TERRAIN, self:resolve('PERMA_WALL', nil, true))
    end
  end

  -- Generate a cave.
  local x0 = 2 + math.floor(self.map.w/2)
  local y0 = 2 + math.floor(self.map.h/2)
  local done = false
  while not done do
    -- *sigh* I wish Lua had a << operator...
    local grd = util.lshift(1, rng.range(1, 4))
    -- Want an average of about 16.
    local roug = rng.range(1, 8) * rng.range(1, 4)
    -- About size/2
    local cutoff = rng.range(1, math.floor(self.map.w/4)) +
		   rng.range(1, math.floor(self.map.h/4)) +
		   rng.range(1, math.floor(self.map.w/4)) +
		   rng.range(1, math.floor(self.map.h/4))

    local hmap = self.generate_hmap(x0, y0, self.map.w, self.map.h, grd, roug, cutoff)
    done = self:generate_fracave(x0, y0, self.map.w, self.map.h, hmap, cutoff, false, true)
  end

  -- Find locations for Eol and the player.
  local eol_spot, plyr_spot = nil, nil
  for x = self.map.w - 1, 2, -1 do
    for y = self.map.h - 1, 2, -1 do
      if self.map:checkEntity(x, y, Map.TERRAIN, 'clean_bold') then
	eol_spot = eol_spot or { type='quest', subtype='eol_start', x=x, y=y }
	plyr_spot = { type='quest', subtype='player_start', x=x, y=y }
      end
    end
  end

  -- Place Eol at his location.
  local eol = self.zone:makeEntityByName(self.level, 'actor', 'EOL_THE_DARK_ELF')
  if eol then
    eol.quest_mon = true
    eol.quest_on_die = 'eol-the-dark-elf'
    self:roomMapAddEntity(eol_spot.x, eol_spot.y, 'actor', eol)
  end

  -- Place up stairs at the player start.
  self.map(plyr_spot.x, plyr_spot.y, Map.TERRAIN, self:resolve('EXIT_GONDOLIN_EOL_QUEST', nil, true))

  -- post_process() will place traps for us.
  return plyr_spot.x, plyr_spot.y, nil, nil, { plyr_spot, eol_spot }
end
