-- $Id: T2LevelFilteredTraps.lua 3096 2017-09-25 23:50:14Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Map = require "engine.Map"
local Generator = require "engine.Generator"

module(..., package.seeall, class.inherit(Generator))

function _M:init(zone, map, level)
  Generator.init(self, zone, map, level)
  self.data = level.data.generator.trap
end

function _M:generate()
  self:regenFrom(1)
end

function _M:regenFrom(current)
  local max
  if self.data.n_traps then
    max = rng.range(self.data.n_traps[1], self.data.n_traps[2])
  else
    -- cf. src/generate.c line 7903 and 7709-7711
    local lev = self.zone:level_adjust_level(self.level)
    local k = util.minBound(math.floor(lev/3), 2, 10)
    max = rng.range(1, k*2)
  end
  for _ = current, max do self:generateOne() end
end

function _M:generateOne(p)
  p = p or {}
  local lev = self.zone:level_adjust_level(self.level)
  -- Hack(TM):  No traps on level 1, unless forced.
  if lev == 1 and not p.force_lvl1 then return end

  local t, x, y
  if p.name then
    t = self.zone:makeEntityByName(self.level, 'trap', p.name)
  else
    -- We count on our level's default trap filter here if the caller
    -- doesn't provide one.
    t = self.zone:makeEntity(self.level, 'trap', p.filter, nil, true)
  end
  if t then
    x, y = p.x, p.y
    if not p.x or not p.y then
      x, y = rng.range(0, self.map.w-1), rng.range(0, self.map.h-1)
      local tries = 0
      while (not self.map:checkEntity(x, y, Map.TERRAIN, 'naked_bold') or
	     self.map:checkEntity(x, y, Map.TERRAIN, 'hasFlags', 'DOOR')) and
	    tries < 10000
      do
	x, y = rng.range(0, self.map.w-1), rng.range(0, self.map.h-1)
	tries = tries + 1
      end
      if tries >= 10000 then x, y = nil, nil end
    end
    if x and y then
      self.zone:addEntity(self.level, t, 'trap', x, y)
    end
  end
end
