-- $Id: EntityTweakable.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'

module(..., package.seeall, class.make)

-- Recognized replaceDisplay() tags, in decreasing priority order.
_M.display_tag_order = { 'mimic', 'hallucination', 'user-pref', 'shimmer' }
_M.display_tags = { mimic=true, hallucination=true, ['user-pref']=true, shimmer=true }

-- A utility method to juggle replace_display Entities from various sources.
function _M:replaceDisplay(tag, rd)
  if not self.display_tags[tag] then
    print(("[ERROR] replaceDisplay() unrecognized tag '%s'"):format(tostring(tag)))
    return
  end

  self.replace_displays = self.replace_displays or {}
  local cur_rd = self.replace_displays[tag]
  if table.equal(cur_rd, rd) then return false end
  self.replace_displays[tag] = rd

  cur_rd = self.replace_display
  local new_rd = nil
  for _, tag in ipairs(self.display_tag_order) do
    new_rd = new_rd or self.replace_displays[tag]
  end
  if table.equal(cur_rd, new_rd) then return false end
  self.replace_display = new_rd
  self:removeAllMOs()
  if self.x and self.y then game.level.map:updateMap(self.x, self.y) end
  return true
end
