-- $Id: FactionsDialog.lua 2919 2017-06-01 02:15:31Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local Checkbox = require 'engine.ui.Checkbox'
local Textzone = require 'engine.ui.Textzone'
local Button = require 'engine.ui.Button'

module(..., package.seeall, class.inherit(Dialog))

local fields = {
  self		= 'Player',
  friend	= 'Pets and friendlies',
  neutral	= 'Neutral NPCs',
  enemy		= 'Enemies',
  danger	= 'Uniques and quest monsters',
}
local field_order = { 'self', 'friend', 'neutral', 'enemy', 'danger' }

function _M:init(title, txt, init_vals)
  Dialog.init(self, title, 300, 300)

  for key, name in pairs(fields) do
    self['c_'..key] = Checkbox.new { title=name, default=init_vals[key] }
  end
  self.c_head = Textzone.new {
    width = self.iw,
    auto_height = true,
    no_color_bleed = true,
    text = txt,
  }
  self.c_foot = Textzone.new {
    width = self.iw,
    auto_height = true,
    no_color_bleed = true,
    text = [[#SLATE#<Ret> to accept, <Esc> to cancel#LAST#]],
  }

  self.c_ok = Button.new {
    text = 'Apply',
    fct = function() self.key:triggerVirtual('ACCEPT') end,
  }
  self.c_cancel = Button.new {
    text = 'Cancel',
    fct = function() self.key:triggerVirtual('EXIT') end,
  }

  local grmbl = {}
  local h = self.c_head.h
  for _, field in ipairs(field_order) do
    grmbl[field] = h
    h = h + self['c_'..field].h
  end
  grmbl.foot = h

  self:loadUI {
    { left=0, top=0, ui=self.c_head },
    { left=0, top=grmbl.self, ui=self.c_self },
    { left=0, top=grmbl.friend, ui=self.c_friend },
    { left=0, top=grmbl.neutral, ui=self.c_neutral },
    { left=0, top=grmbl.enemy, ui=self.c_enemy },
    { left=0, top=grmbl.danger, ui=self.c_danger },
    { left=0, top=grmbl.foot, ui=self.c_foot },
    { left=0, bottom=0, ui=self.c_ok },
    { left=self.c_ok.w + 5, bottom=0, ui=self.c_cancel },
  }
  self:setupUI(true, true)

  self.key:addBinds {
    ACCEPT = function()
      self:setupValues()
      game:unregisterDialog(self)
    end,
    EXIT = function() game:unregisterDialog(self) end,
  }
end

function _M:setupValues()
  self.value = {}
  for k, _ in pairs(fields) do self.value[k] = self['c_'..k].checked end
end

function _M:get_value()
  return self.value
end

function _M:get_display_val()
  return self:displayValue(self.value)
end

function _M:displayValue(val)
  if type(val) ~= 'table' then return '???' end
  local disp = {}
  for _, k in ipairs(field_order) do
    if val[k] then
      disp[#disp+1] = k == 'danger' and 'unique' or k
    end
  end
  if #disp == 0 then return 'none' end
  if #disp <= 2 then return table.concat(disp, '+') end
  return ('%d factions'):format(#disp)
end
