-- $Id: InertiaControlDialog.lua 3107 2017-09-26 23:34:29Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require "engine.class"
local Dialog = require "engine.ui.Dialog"
local List = require 'engine.ui.List'

module(..., package.seeall, class.inherit(Dialog))

function _M:init(actor, max_power)
  self.actor = actor
  self.max_power = max_power

  Dialog.init(self, 'Select Inertia Control Spell', 1, 1)

  self:generateList()
  self.c_list = List.new {
    width = self.max_w + 10,
    nb_items = 10,
    scrollbar = true,
    list = self.list,
    select = function(item, sel) self:talentTooltip(item) end,
    fct = function(item, sel) self:use(item) end,
  }

  self:loadUI {
    { left=0, top=0, ui=self.c_list },
  }
  self:setFocus(self.c_list)
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function()
      self.actor:talentDialogReturn(false)
      game:unregisterDialog(self)
    end
  }
end

function _M:talentTooltip(item)
  if not item.t then return end
  local text = tstring {
    {'color','GOLD'}, {'font','bold'}, item.t.name,
    {'font','normal'}, {'color','LAST'}, true
  }
  text:merge(self.actor:getTalentFullDescription(item.t))
  game:tooltipDisplayAtMap(game.w, game.h, tostring(text))
end

function _M:use(item)
  if not item.t or item.t.inertia > self.max_power then return end
  self.selected_spell = item.t.id
  game:unregisterDialog(self)
  self.actor:talentDialogReturn(true)
end

function _M:generateList()
  local list = {}
  self.max_w = 0

  for _, t in pairs(self.actor.talents_def) do
    if self.actor:knowTalent(t.id) and t.is_spell and t.inertia then
      local name = t.name
      if t.inertia[2] > self.max_power then name = '#GREY#'..name..'#LAST#' end
      list[#list+1] = { name=name, t=t }
      local w, _ = self.font:size(t.name)
      self.max_w = math.max(self.max_w, w)
    end
  end

  self.list = list
end
