-- $Id: ListTalentsDialog.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require "engine.class"
local Dialog = require "engine.ui.Dialog"
local List = require "engine.ui.List"

module(..., package.seeall, class.inherit(Dialog))

function _M:init(title, actor, ids)
  self.actor = actor
  self.ids = ids

  self:generateList()
  Dialog.init(self, title, 1, 1)

  local w = self.font_bold:size(title)
  self.c_list = List.new {
    width = math.max(w, self.max_w) + 10,
    nb_items = #self.list,
    list = self.list,
    select = function(item, sel) self:talentTooltip(item) end,
    fct = function(item, sel) end,
  }

  self:loadUI { { left=0, top=0, ui=self.c_list } }
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end
  }
end

function _M:on_register()
  self:talentTooltip(self.list[1])
end

function _M:talentTooltip(item)
  local text = tstring {
    {'color','GOLD'}, {'font','bold'}, item.t.name,
    {'font','normal'}, {'color','LAST'}, true
  }
  text:merge(self.actor:getTalentFullDescription(item.t))
  game:tooltipDisplayAtMap(game.w, game.h, tostring(text))
end

function _M:generateList()
  local list = {}
  self.max_w = 0

  for _, tid in ipairs(self.ids) do
    local t = self.actor:getTalentFromId(tid)
    if t then
      list[#list+1] = { name=t.name, t=t }
      local w, _ = self.font:size(t.name)
      self.max_w = math.max(self.max_w, w)
    end
  end

  self.list = list
end
