-- $Id: ShowEquipment.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local ShowEquipment = require 'engine.dialogs.ShowEquipment'

module(..., package.seeall, class.inherit(ShowEquipment))

function _M:init(title, actor, filter, action)
  ShowEquipment.init(self, title, actor, filter, action)

  -- Widen the 'char' column a bit to stop the @#$%ing annoying wiggling.
  self.c_list:setColumns {
    {name="", width={26,"fixed"}, display_prop="char"},
    {name="", width={24,"fixed"}, display_prop="object", direct_draw=function(item, x, y) if item.object then item.object:toScreen(nil, x+4, y, 16, 16) end end},
    {name="Equipment", width=72, display_prop="name"},
    {name="Category", width=20, display_prop="cat"},
    {name="Enc.", width=8, display_prop="encumberance"},
  }
end

function _M:generateList()
  ShowEquipment.generateList(self)

  -- Slight Hack(TM):  Hide any equipment slots that are supposed to be
  -- hidden.
  local list = { chars={} }
  for _, l in ipairs(self.list) do
    local def = self.actor.inven_def[l.inven]
    local hide = util.getval(self.infos and self.infos.hide, self.actor)
    if not hide then
      list[#list+1] = l
      l.id = #list
      if l.char ~= '' then list.chars[l.char] = #list end
    end
  end
  self.list = list
  self.c_list:setList(self.list)
end

function _M:inventoryLiveUpdate()
  self:generateList()
end
