-- $Id: DebugDialog.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local List = require 'engine.ui.List'
local Textzone = require 'engine.ui.Textzone'
local Map = require 'mod.class.Map'

module(..., package.seeall, class.inherit(Dialog))

function _M:init()
  self:generateList()
  Dialog.init(self, 'Debug/Testing', 1, 1)

  local text = Textzone.new {
    width = 400,
    auto_height = true,
    no_color_bleed = true,
    text = [[This dialog provides low-level access for debugging and development testing purposes.  Messing around with it is liable to break something if you don't know #{italic}#exactly#{normal}# what you're doing.]],
  }
  local list = List.new {
    width = 400,
    height = 500,
    list = self.list,
    fct = function(item) self:use(item) end,
  }

  self:loadUI {
    { left=0, top=0, ui=text },
    { left=0, bottom=0, ui=list },
  }
  self:setupUI(true, true)
  self:setFocus(list)

  self.key:addCommands {
    __TEXTINPUT = function(c)
      if self.list and self.list.chars[c] then
	self:use(self.list[self.list.chars[c]])
      end
    end
  }
  self.key:addBinds { EXIT = function() game:unregisterDialog(self) end }
end

function _M:on_register()
  game:onTickEnd(function() self.key:unicodeInput(true) end)
end

function _M:use(item)
  if not item then return end
  game:unregisterDialog(self)

  if item.dialog then
    local d = require('mod.dialogs.debug.'..item.dialog).new()
    game:registerDialog(d)
  elseif item.action then
    item.action()
  end
end

function _M:generateList()
  local list = {
    { name = 'Create Item', dialog = 'CreateItemDialog' },
    { name = 'Change Zone', dialog = 'ChangeZoneDialog' },
    { name = 'Find Unique', dialog = 'FindUniqueDialog' },
    { name = 'Test Item Generation', dialog = 'GenerateItemsDialog' },
    { name = 'Wizard-lite level', action = function()
	local map = game.level.map
	map:liteAll(0, 0, map.w, map.h)
	map:rememberAll(0, 0, map.w, map.h)
      end
    },
    { name = 'Detect all traps on level', action = function()
	local map = game.level.map
	local n = 0
	for x = 0, map.w - 1 do
	  for y = 0, map.h - 1 do
	    local t = map(x, y, Map.TRAP)
	    if t then
	      t:setKnown(game.player, true)
	      game.level.map:updateMap(x, y)
	      game.level.map.remembers(x, y, true)
	      n = n + 1
	    end
	  end
	end
	game.log(('%d trap%s detected'):format(n, n == 1 and '' or 's'))
      end
    },
    { name = 'Forget all traps on level', action = function()
	local map = game.level.map
	local n = 0
	for x = 0, map.w - 1 do
	  for y = 0, map.h - 1 do
	    local t = map(x, y, Map.TRAP)
	    if t then
	      t:setKnown(game.player, false)
	      game.level.map:updateMap(x, y)
	      n = n + 1
	    end
	  end
	end
	game.log(('%d trap%s forgotten'):format(n, n == 1 and '' or 's'))
      end
    },
    chars = {},
  }
  self:triggerHook{ 'DebugDialog:generate', menu=list }

  for i, v in ipairs(list) do
    local c = self:makeKeyChar(i)
    v.name = c..') '..v.name
    list.chars[c] = i
  end
  self.list = list
end
